from flask_wtf import FlaskForm
from wtforms import RadioField, BooleanField, FloatField, IntegerField, SubmitField, FormField
from wtforms.validators import DataRequired

class BeginForm(FlaskForm):
    submit = SubmitField('Begin')

class PatternForm(FlaskForm):
    flexure_type    = RadioField('Flexure type', choices=[
        ('let','LET'),
        ('coil','Coil'),
        ('switchback','Switchback'),
        ('ydx','YdX')
        ], default='let', validators=[DataRequired()])
    wallpaper_group = RadioField('Wallpaper group', choices=[
        ('cmm','cmm/pmm'),
        ('p4','p4'),
        ('p4g','p4g'),
        ('p4m','p4m')
        ], default='cmm', validators=[DataRequired()])
    back = SubmitField('Back')
    submit = SubmitField('Next')

    def validate(self):
        validDict = {                      #For checking if flexure type and wallpaper group are compatible
                'let':          ['cmm', 'p4m'],
                'switchback':   ['cmm', 'p4', 'p4g'],
                'coil':         ['cmm', 'p4'],
                'ydx':          ['cmm'],
                'solid':        ['cmm', 'p4', 'p4m', 'p4g']
        }
        if not FlaskForm.validate(self):
            return False
        result = True
        if not self.wallpaper_group.data in validDict[self.flexure_type.data]:
            self.wallpaper_group.errors.append('Flexure type and wallpaper group are not compatible.')
            result = False
        return result

class ModeForm(FlaskForm):
    sample  = BooleanField('Sample')
    full    = BooleanField('Full SVG')
    abaqus  = BooleanField('Abaqus')

    def validate(self):
        if not FlaskForm.validate(self):
            return False
        result = True
        if not (self.sample.data or self.full.data or self.abaqus.data):
            self.abaqus.errors.append('Must choose at least one mode.')
            result = False
        return result

class PartForm(FlaskForm):
    gen_reg     = BooleanField('Generating region')
    unit_cell   = BooleanField('Unit cell')
    flex_pat    = BooleanField('Flexure pattern')

    def validate(self):
        if not FlaskForm.validate(self):
            return False
        result = True
        if not (self.gen_reg.data or self.unit_cell.data or self.flex_pat.data):
            self.flex_pat.errors.append('Must choose at least one part.')
            result = False
        return result

class OptionForm(FlaskForm):
    mode = FormField(ModeForm)
    part = FormField(PartForm)
    unit =  RadioField('Unit', choices=[
        ('none','none'),
        ('mm','mm'),
        ('in','inches')
        ], default='none', validators=[DataRequired()])
    cut =   RadioField('Cut type', choices=[
        ('gaps','Gaps'),
        ('slits','Slits')
        ], default='gaps', validators=[DataRequired()])
    back = SubmitField('Back')
    submit = SubmitField('Next')

class ParameterForm(FlaskForm):
    rotation        = FloatField('Rotation',        default=0.0)
    stroke_width    = FloatField('Stroke width',    default=0.025)
    thickness       = FloatField('Thickness',       default=0.0)
    partition       = RadioField('Partition', choices=[
        ('yes','Yes'),
        ('no','No')
        ], default='no')
    height          = FloatField('Height',          default=50.0)
    width           = FloatField('Width',           default=50.0)
    cut_width       = FloatField('Cut width',       default=0.5)
    edge_space      = FloatField('Edge space',      default=1.0)
    flexure_length  = FloatField('Flexure length',  default=5.0)
    flexure_width   = FloatField('Flexure width',   default=1.0)
    inside_start    = RadioField('Start', choices=[
        ('outside','Outside'),
        ('inside','Inside')
        ], default='outside')
    junction_length = FloatField('Junction length', default=2.0)
    num_flex        = IntegerField('Number of flexures', default=3)
    num_turns       = IntegerField('Number of turns', default=1)
    side_cut        = FloatField('Side cut',        default=-1.0)
    solid_width     = FloatField('Solid width',     default=2.5)
    stem_length     = FloatField('Stem length',     default=1.0)
    stem_width      = FloatField('Stem width',      default=1.0)
    thetaDeg        = FloatField('Angle',           default=45.0)
    back            = SubmitField('Back')
    submit          = SubmitField('Submit')