import os

def makeAbaqusString(gen_reg_coords, unit_coords, name, part, thickness, junction, nx, ny, xmin, ymin, xmax, ymax):
    pattern = name.replace(".py","")
    width = xmax - xmin
    height = ymax - ymin
    size = max(width, height)
    string_begin_model = (
        """from abaqus import *                                                                              \n"""
        """from abaqusConstants import *                                                                     \n"""
        """session.Viewport(name='Viewport: 1', origin=(0.0, 0.0), width={0}, height={1})                    \n"""
        """session.viewports['Viewport: 1'].makeCurrent()                                                    \n"""
        """session.viewports['Viewport: 1'].maximize()                                                       \n"""
        """from caeModules import *                                                                          \n"""
        """from driverUtils import executeOnCaeStartup                                                       \n"""
        """executeOnCaeStartup()                                                                             \n"""
        """session.viewports['Viewport: 1'].partDisplay.geometryOptions.setValues(referenceRepresentation=ON)\n"""
        """Mdb()                                                                                             \n"""
        """session.viewports['Viewport: 1'].setValues(displayedObject=None)                                  \n"""
        """mdb.Model(name='{3}', modelType=STANDARD_EXPLICIT)                                                \n"""
        """session.viewports['Viewport: 1'].setValues(displayedObject=None)                                  \n"""
        """s = mdb.models['{3}'].ConstrainedSketch(name='__profile__', sheetSize={2})                        \n"""
        """g, v, d, c = s.geometry, s.vertices, s.dimensions, s.constraints                                  \n"""
        """s.setPrimaryObject(option=STANDALONE)                                                             \n"""
        ).format(200,200,size,pattern)
    string_create_model = ""
    if thickness > 0:
        string_thickness = """p.BaseSolidExtrude(sketch=s, depth={0})""".format(thickness)
    else:
        string_thickness = """p.BaseShell(sketch=s)"""
    string_end_model = (
        """p = mdb.models['{1}'].Part(name='Unit', dimensionality=THREE_D, type=DEFORMABLE_BODY)\n"""
        """p = mdb.models['{1}'].parts['Unit']                                                  \n"""
        """{0}                                                                                  \n"""
        """s.unsetPrimaryObject()                                                               \n"""
        """p = mdb.models['{1}'].parts['Unit']                                                  \n"""
        """session.viewports['Viewport: 1'].setValues(displayedObject=p)                        \n"""
        """del mdb.models['{1}'].sketches['__profile__']                                        \n"""
        ).format(string_thickness,pattern)
    #Create the unit cell
    for poly_coords in unit_coords:
        i = 1
        while i < len(poly_coords):
            string_create_model += "s.Line(point1=({0}, {1}), point2=({2}, {3}))\n".format(poly_coords[i-1][0], poly_coords[i-1][1], poly_coords[i][0], poly_coords[i][1])
            i += 1
        string_create_model += "s.Line(point1=({0}, {1}), point2=({2}, {3}))\n".format(poly_coords[i-1][0], poly_coords[i-1][1], poly_coords[0][0], poly_coords[0][1])
    string_create_partitions = ""
    #Create partitions on unit cell
    if part:
        if thickness > 0:
            string_create_partitions = (
                """p = mdb.models['{9}'].parts['Unit']                                                            \n"""
                """p.DatumPointByCoordinate(coords=({1}, {5}, {6}))                                               \n"""
                """p = mdb.models['{9}'].parts['Unit']                                                            \n"""
                """p.DatumPointByCoordinate(coords=({1}, {3}, {6}))                                               \n"""
                """p = mdb.models['{9}'].parts['Unit']                                                            \n"""
                """p.DatumPointByCoordinate(coords=({1}, {3}, {8}))                                               \n"""
                """p = mdb.models['{9}'].parts['Unit']                                                            \n"""
                """p.DatumPointByCoordinate(coords=({2}, {4}, {6}))                                               \n"""
                """p = mdb.models['{9}'].parts['Unit']                                                            \n"""
                """p.DatumPointByCoordinate(coords=({0}, {4}, {6}))                                               \n"""
                """p = mdb.models['{9}'].parts['Unit']                                                            \n"""
                """p.DatumPointByCoordinate(coords=({0}, {4}, {8}))                                               \n"""
                """p = mdb.models['{9}'].parts['Unit']                                                            \n"""
                """p.DatumPointByCoordinate(coords=({0}, {5}, {7}))                                               \n"""
                """p = mdb.models['{9}'].parts['Unit']                                                            \n"""
                """p.DatumPointByCoordinate(coords=({0}, {3}, {7}))                                               \n"""
                """p = mdb.models['{9}'].parts['Unit']                                                            \n"""
                """p.DatumPointByCoordinate(coords=({2}, {3}, {7}))                                               \n"""
                """p = mdb.models['{9}'].parts['Unit']                                                            \n"""
                """c = p.cells                                                                                    \n"""
                """pickedCells = c.getSequenceFromMask(mask=('[#1 ]', ), )                                        \n"""
                """v1, e, d1 = p.vertices, p.edges, p.datums                                                      \n"""
                """p.PartitionCellByPlaneThreePoints(point1=d1[2], point2=d1[3], point3=d1[4], cells=pickedCells) \n"""
                """p = mdb.models['{9}'].parts['Unit']                                                            \n"""
                """c = p.cells                                                                                    \n"""
                """pickedCells = c.getSequenceFromMask(mask=('[#3 ]', ), )                                        \n"""
                """v2, e1, d2 = p.vertices, p.edges, p.datums                                                     \n"""
                """p.PartitionCellByPlaneThreePoints(point1=d2[5], point2=d2[6], point3=d2[7], cells=pickedCells) \n"""
                """p = mdb.models['{9}'].parts['Unit']                                                            \n"""
                """c = p.cells                                                                                    \n"""
                """pickedCells = c.getSequenceFromMask(mask=('[#f ]', ), )                                        \n"""
                """v1, e, d1 = p.vertices, p.edges, p.datums                                                      \n"""
                """p.PartitionCellByPlaneThreePoints(point1=d1[8], point2=d1[9], point3=d1[10], cells=pickedCells)\n""" 
                ).format(xmin, (xmin+xmax)/2, xmax, ymin, (ymin+ymax)/2, ymax, 0.0, thickness/2, thickness, pattern)
        else:
            string_create_partitions = (
                """p = mdb.models['{6}'].parts['Unit']                                         \n"""
                """p.DatumPointByCoordinate(coords=({1}, {3}, 0.0))                            \n"""
                """p = mdb.models['{6}'].parts['Unit']                                         \n"""
                """p.DatumPointByCoordinate(coords=({1}, {5}, 0.0))                            \n"""
                """p = mdb.models['{6}'].parts['Unit']                                         \n"""
                """p.DatumPointByCoordinate(coords=({0}, {4}, 0.0))                            \n"""
                """p = mdb.models['{6}'].parts['Unit']                                         \n"""
                """p.DatumPointByCoordinate(coords=({2}, {4}, 0.0))                            \n"""
                """p = mdb.models['{6}'].parts['Unit']                                         \n"""
                """f = p.faces                                                                 \n"""
                """pickedFaces = f.getSequenceFromMask(mask=('[#1 ]', ), )                     \n"""
                """v1, e, d1 = p.vertices, p.edges, p.datums                                   \n"""
                """p.PartitionFaceByShortestPath(point1=d1[2], point2=d1[3], faces=pickedFaces)\n"""
                """p = mdb.models['{6}'].parts['Unit']                                         \n"""
                """f = p.faces                                                                 \n"""
                """pickedFaces = f.getSequenceFromMask(mask=('[#3 ]', ), )                     \n"""
                """v2, e1, d2 = p.vertices, p.edges, p.datums                                  \n"""
                """p.PartitionFaceByShortestPath(point1=d2[4], point2=d2[5], faces=pickedFaces)\n"""
                ).format(xmin, (xmin+xmax)/2, xmax, ymin, (ymin+ymax)/2, ymax, pattern)
    string_merge = ""
    x = 1
    while x <= nx:
        y = 1
        while y <= ny:
            string_merge += "a.instances['Unit-1-lin-{0}-{1}'], ".format(x,y)
            y += 1
        x += 1
    string_merge = string_merge.replace("a.instances['Unit-1-lin-1-1'], ","a.instances['Unit-1'], ")
    string_merge += ")"
    if part:
        string_merge += ", keepIntersections=ON"
    #Creates flexure pattern by patterning the unit cell and merging
    string_create_assembly = (
        """a = mdb.models['{5}'].rootAssembly                                                                                                                              \n"""
        """session.viewports['Viewport: 1'].setValues(displayedObject=a)                                                                                                   \n"""
        """session.viewports['Viewport: 1'].assemblyDisplay.setValues(optimizationTasks=OFF, geometricRestrictions=OFF, stopConditions=OFF)                                \n"""
        """a = mdb.models['{5}'].rootAssembly                                                                                                                              \n"""
        """a.DatumCsysByDefault(CARTESIAN)                                                                                                                                 \n"""
        """p = mdb.models['{5}'].parts['Unit']                                                                                                                             \n"""
        """a.Instance(name='Unit-1', part=p, dependent=OFF)                                                                                                                \n"""
        """a = mdb.models['{5}'].rootAssembly                                                                                                                              \n"""
        """a.LinearInstancePattern(instanceList=('Unit-1', ), direction1=(1.0, 0.0, 0.0), direction2=(0.0, 1.0, 0.0), number1={1}, number2={2}, spacing1={3}, spacing2={4})\n"""
        """a = mdb.models['{5}'].rootAssembly                                                                                                                              \n"""
        """a.InstanceFromBooleanMerge(name='Pattern', instances=({0}, originalInstances=DELETE, domain=GEOMETRY)                                                           \n"""
        """p = mdb.models['{5}'].parts['Unit']                                                                                                                             \n"""
        """a.Instance(name='Unit-1', part=p, dependent=OFF)                                                                                                                \n"""
        """a.features['Unit-1'].suppress()                                                                                                                                 \n"""
        """p = mdb.models['{5}'].parts['Pattern']                                                                                                                          \n"""
        """session.viewports['Viewport: 1'].setValues(displayedObject=p)                                                                                                   \n"""
        ).format(string_merge, nx, ny, width, height, pattern)
    string_begin_gen_reg = (
        """s = mdb.models['{0}'].ConstrainedSketch(name='__profile__', sheetSize={1})                        \n"""
        """g, v, d, c = s.geometry, s.vertices, s.dimensions, s.constraints                                  \n"""
        """s.setPrimaryObject(option=STANDALONE)                                                             \n"""
        ).format(pattern,size)
    string_create_gen_reg = ""
    if thickness > 0:
        string_thickness = """p.BaseSolidExtrude(sketch=s, depth={0})""".format(thickness/2)
    else:
        string_thickness = """p.BaseShell(sketch=s)"""
    string_end_gen_reg = (
        """a = mdb.models['{1}'].rootAssembly                                                       \n"""
        """p = mdb.models['{1}'].Part(name='Gen_reg', dimensionality=THREE_D, type=DEFORMABLE_BODY) \n"""
        """p = mdb.models['{1}'].parts['Gen_reg']                                                   \n"""
        """{0}                                                                                      \n"""
        """s.unsetPrimaryObject()                                                                   \n"""
        """p = mdb.models['{1}'].parts['Gen_reg']                                                   \n"""
        """session.viewports['Viewport: 1'].setValues(displayedObject=p)                            \n"""
        """del mdb.models['{1}'].sketches['__profile__']                                            \n"""
        """a.Instance(name='Gen_reg-1', part=p, dependent=OFF)                                      \n"""
        """session.viewports['Viewport: 1'].assemblyDisplay.hideInstances(instances=('Gen_reg-1', ))\n"""
        """a.features['Gen_reg-1'].suppress()                                                       \n"""
        """p1 = mdb.models['{1}'].parts['Pattern']                                                  \n"""
        """session.viewports['Viewport: 1'].setValues(displayedObject=p1)                           \n"""
        ).format(string_thickness,pattern)
    #Creates the generating region
    for poly_coords in gen_reg_coords:
        i = 1
        while i < len(poly_coords):
            string_create_gen_reg += "s.Line(point1=({0}, {1}), point2=({2}, {3}))\n".format(poly_coords[i-1][0], poly_coords[i-1][1], poly_coords[i][0], poly_coords[i][1])
            i += 1
        string_create_model += "s.Line(point1=({0}, {1}), point2=({2}, {3}))\n".format(poly_coords[i-1][0], poly_coords[i-1][1], poly_coords[0][0], poly_coords[0][1])
    return string_begin_model + string_create_model + string_end_model + string_create_partitions + string_create_assembly + string_begin_gen_reg + string_create_gen_reg + string_end_gen_reg

def createAbaqus(gen_reg_coords, unit_coords, path, name, part, thickness, junction, nx, ny, xmin, ymin, xmax, ymax):
    abaqus_string = makeAbaqusString(gen_reg_coords, unit_coords, name, part, thickness, junction, nx, ny, xmin, ymin, xmax, ymax)
    filepath = path+"\\"+name
    file=open(filepath,'w')
    file.write(abaqus_string)
    file.close()
    return