import os
import io
import pathlib
import zipfile
from flask import render_template, flash, redirect, url_for, Response, current_app, session, send_file
from app import app
from app.forms import BeginForm, PatternForm, OptionForm, ParameterForm
from app.patternGeneratorsClass import generatePattern
import uuid

@app.route('/', methods=['GET', 'POST'])
@app.route('/index', methods=['GET', 'POST'])
def index():
    form = BeginForm()
    if form.validate_on_submit():
        return redirect(url_for('pattern'))
    return render_template('index.html', form=form)

@app.route('/pattern', methods=['GET', 'POST'])
def pattern():
    form = PatternForm()
    if not form.back.data:
        if form.validate_on_submit():
            if form.submit.data:
                session['flexure_type'] = form.flexure_type.data
                session['wallpaper_group'] = form.wallpaper_group.data
                return redirect(url_for('option'))
        else:
            return render_template('pattern.html', form=form)
    else:
        return redirect(url_for('index'))

@app.route('/option', methods=['GET', 'POST'])
def option():
    form = OptionForm()
    if not form.back.data:
        if form.validate_on_submit():
            if form.submit.data:
                session['mode'] = form.mode.data
                session['part'] = form.part.data
                session['unit'] = form.unit.data
                session['cut']  = form.cut.data
                return redirect(url_for('parameter'))
        else:
            return render_template('option.html', form=form)
    else:
        return redirect(url_for('pattern'))

@app.route('/parameter', methods=['GET', 'POST'])
def parameter():
    form = ParameterForm()
    ft          = session.get('flexure_type')
    wg          = session.get('wallpaper_group')
    sample      = session.get('mode').get('sample')
    full        = session.get('mode').get('full')
    abaqus      = session.get('mode').get('abaqus')
    gen_reg     = session.get('part').get('gen_reg')
    unit_cell   = session.get('part').get('unit_cell')
    flex_pat    = session.get('part').get('flex_pat')
    cut         = session.get('cut')

    if not form.back.data:
        if form.validate_on_submit():
            if form.submit.data:
                session['ID'] = uuid.uuid4()
                session['rotation']         = form.rotation.data
                session['stroke_width']     = form.stroke_width.data
                session['thickness']        = form.thickness.data
                session['partition']        = form.partition.data
                session['height']           = form.height.data
                session['width']            = form.width.data
                session['cut_width']        = form.cut_width.data
                session['edge_space']       = form.edge_space.data
                session['flexure_length']   = form.flexure_length.data
                session['flexure_width']    = form.flexure_width.data
                session['inside_start']     = form.flexure_width.data
                session['junction_length']  = form.junction_length.data
                session['num_flex']         = form.num_flex.data
                session['num_turns']        = form.num_turns.data
                session['side_cut']         = form.side_cut.data
                session['solid_width']      = form.solid_width.data
                session['stem_length']      = form.stem_length.data
                session['stem_width']       = form.stem_width.data
                session['thetaDeg']         = form.thetaDeg.data

                para_dict = {}

                para_dict['flexure_type'] = session.get('flexure_type')
                para_dict['wallpaper_group'] = session.get('wallpaper_group')
                
                para_dict['sample']     = session.get('mode').get('sample')
                para_dict['full']       = session.get('mode').get('full')
                para_dict['abaqus']     = session.get('mode').get('abaqus')
                para_dict['gen_reg']    = session.get('part').get('gen_reg')
                para_dict['unit_cell']  = session.get('part').get('unit_cell')
                para_dict['flex_pat']   = session.get('part').get('flex_pat')
                if session.get('unit') == 'none':
                    para_dict['unit'] = ''
                else:
                    para_dict['unit'] = session.get('unit')
                if session.get('cut') == 'gaps':
                    para_dict['cut'] = True
                elif session.get('cut') == 'slits':
                    para_dict['cut'] = False
                else:
                    para_dict['cut'] = session.get('cut')

                para_dict['rotation']       = session.get('rotation')
                para_dict['stroke_width']   = session.get('stroke_width')
                para_dict['thickness']      = session.get('thickness')
                if session.get('partition') == 'yes':
                    para_dict['partition']  = True
                else:
                    para_dict['partition']  = False
                para_dict['height']         = session.get('height')
                para_dict['width']          = session.get('width')
                para_dict['cut_width']      = session.get('cut_width')
                para_dict['edge_space']     = session.get('edge_space')
                para_dict['flexure_length'] = session.get('flexure_length')
                para_dict['flexure_width']  = session.get('flexure_width')
                if session.get('inside_start') == 'inside':
                    para_dict['inside_start'] = False #In the program, "inside start" refers to the stem starting on the inside rather than the cut
                else:
                    para_dict['inside_start'] = True
                para_dict['junction_length']= session.get('junction_length')
                para_dict['num_flex']       = session.get('num_flex')
                para_dict['num_turns']      = session.get('num_turns')
                if session.get('side_cut') >= 0:
                    para_dict['side_cut']   = session.get('side_cut')
                else:
                    para_dict['side_cut']   = 'default'
                para_dict['solid_width']    = session.get('solid_width')
                para_dict['stem_length']    = session.get('stem_length')
                if session.get('stem_width') >= 0:
                    para_dict['stem_width'] = session.get('stem_width')
                else:
                    para_dict['stem_width'] = 'default'
                para_dict['thetaDeg']       = session.get('thetaDeg')

                filepath = "{0}\\{1}\\".format(os.getcwd(),session.get('ID'))
                generatePattern(filepath, para_dict)
                basepath = pathlib.Path("./{}".format(session.get('ID')))
                data = io.BytesIO()
                with zipfile.ZipFile(data, mode='w') as z:
                    for f_name in basepath.iterdir():
                        z.write(f_name)
                data.seek(0)
                return send_file(
                    data,
                    mimetype='application/zip',
                    as_attachment=True,
                    attachment_filename='flexure_pattern.zip'
                )
        else:
            return render_template('parameter.html', ft=ft, wg=wg, form=form, sample=sample, full=full, abaqus=abaqus, gen_reg=gen_reg, unit_cell=unit_cell, flex_pat=flex_pat, cut=cut)
    else:
        return redirect(url_for('option'))